package com.gxzc.zen.web.sys.util

import com.github.benmanes.caffeine.cache.Caffeine
import com.gxzc.zen.common.dto.UploadFileInfo
import com.gxzc.zen.common.dto.ZenFileMetadata
import org.apache.commons.io.FilenameUtils
import org.springframework.cache.caffeine.CaffeineCache
import java.io.File
import java.util.*
import java.util.concurrent.TimeUnit

/**
 *
 * @author NorthLan
 * @date 2018/7/3
 * @url https://noahlan.com
 */
@Suppress("UNCHECKED_CAST")
object UploadCacheUtil {
    private val batchFileMap = CaffeineCache("batchFileMap", Caffeine.newBuilder().expireAfterWrite(1L, TimeUnit.HOURS).build(), true) // 文件批量上传批次实际文件信息

    private fun getBatchFileList(batchId: String): MutableList<UploadFileInfo>? {
        return batchFileMap[batchId]?.get() as? MutableList<UploadFileInfo>
    }

    private fun addFileInfo(fileInfo: UploadFileInfo, batchId: String) {
        var fileList = getBatchFileList(batchId)
        if (fileList == null) {
            fileList = mutableListOf()
            batchFileMap.put(batchId, fileList)
        }
        fileList.add(fileInfo)
    }

    /**
     * 添加文件数据
     */
    fun addUploadFileInfo(fileMetadata: ZenFileMetadata, file: File) {
        val entity = UploadFileInfo().apply {
            // filename
            this.filename = if (fileMetadata.rename.isNullOrEmpty()) {
                fileMetadata.filename
            } else {
                "${FilenameUtils.removeExtension(fileMetadata.rename)}.${FilenameUtils.getExtension(fileMetadata.filename)}"
            }
            this.realFilename = file.name
            this.originFilename = fileMetadata.filename
            // infomation
            this.relativePath = if (fileMetadata.repath.isNullOrEmpty()) fileMetadata.relativePath?.replace(this.originFilename!!, "") else fileMetadata.repath
            this.lastModified = Date(file.lastModified())
            this.md5 = fileMetadata.md5
            this.extension = FilenameUtils.getExtension(file.name)
            this.filesize = fileMetadata.totalSize
        }
        addFileInfo(entity, fileMetadata.batchId!!)
    }

    /**
     * 获取某上传批次所有文件数据
     */
    fun getUploadInfosByBatchId(batchId: String): MutableList<UploadFileInfo> {
        val ret = getBatchFileList(batchId) ?: mutableListOf()
        ret.forEach {
            generateURL(it)
        }
        return ret
    }

    /**
     * 清理缓存
     */
    fun evictCache(batchId: String) {
        batchFileMap.evict(batchId)
    }

    private fun generateURL(it: UploadFileInfo?) {
        if (it != null) {
            it.url = "/${it.realFilename}"
        }
    }
}